@@ -9,7 +9,7 @@ mod parser {
             panic!(
                 "Unexpected success while parsing:\n{}\n{:#?}",
                 source,
-                ast.0.nodes()
+                ast.nodes()
             );
         }
     }
@@ -20,7 +20,7 @@ mod parser {
             panic!(
                 "Unexpected success while parsing:\n{}\n{:#?}",
                 source,
-                ast.0.nodes()
+                ast.nodes()
             );
         }
     }
@@ -54,16 +54,6 @@ mod parser {
                 check_parsing_fails(source);
             }
 
-            #[test]
-            fn extra_indentation_in_arithmetic() {
-                let source = "
-x = 1
-    + 2
-      + 3
-";
-                check_parsing_fails(source);
-            }
-
             #[test]
             fn list_end_with_incorrect_indentation() {
                 let source = "
